from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.team_right import TeamRight


T = TypeVar("T", bound="TeamRole")


@_attrs_define
class TeamRole:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        rights (Union[Unset, List['TeamRight']]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    rights: Union[Unset, List["TeamRight"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        rights: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rights, Unset):
            rights = []
            for rights_item_data in self.rights:
                rights_item = rights_item_data.to_dict()
                rights.append(rights_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if rights is not UNSET:
            field_dict["rights"] = rights

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.team_right import TeamRight

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        rights = []
        _rights = d.pop("rights", UNSET)
        for rights_item_data in _rights or []:
            rights_item = TeamRight.from_dict(rights_item_data)

            rights.append(rights_item)

        team_role = cls(
            id=id,
            name=name,
            rights=rights,
        )

        team_role.additional_properties = d
        return team_role

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateCustomUserRolePayload")


@_attrs_define
class CreateCustomUserRolePayload:
    """
    Attributes:
        name (str): Name of custom user role
        extended_role (Union[Unset, str]): Custom role. Must not be one of the defined values (i.e. "user", "observer",
            "stakeholder")
        granted_rights (Union[Unset, List[str]]): Rights granted to the custom user role.
        disallowed_rights (Union[Unset, List[str]]): Rights disallowed for the custom user role.
    """

    name: str
    extended_role: Union[Unset, str] = UNSET
    granted_rights: Union[Unset, List[str]] = UNSET
    disallowed_rights: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        extended_role = self.extended_role

        granted_rights: Union[Unset, List[str]] = UNSET
        if not isinstance(self.granted_rights, Unset):
            granted_rights = self.granted_rights

        disallowed_rights: Union[Unset, List[str]] = UNSET
        if not isinstance(self.disallowed_rights, Unset):
            disallowed_rights = self.disallowed_rights

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if extended_role is not UNSET:
            field_dict["extendedRole"] = extended_role
        if granted_rights is not UNSET:
            field_dict["grantedRights"] = granted_rights
        if disallowed_rights is not UNSET:
            field_dict["disallowedRights"] = disallowed_rights

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        extended_role = d.pop("extendedRole", UNSET)

        granted_rights = cast(List[str], d.pop("grantedRights", UNSET))

        disallowed_rights = cast(List[str], d.pop("disallowedRights", UNSET))

        create_custom_user_role_payload = cls(
            name=name,
            extended_role=extended_role,
            granted_rights=granted_rights,
            disallowed_rights=disallowed_rights,
        )

        create_custom_user_role_payload.additional_properties = d
        return create_custom_user_role_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import re
import httpx
import semver
from textual import on, work
from textual.app import ComposeResult
from textual.containers import Horizontal
from textual.css.query import NoMatches
from textual.message import Message
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import Button, Select
import tygenie.logger as ty_logger

from tygenie import consts
import tygenie.opsgenie as opsgenie
from tygenie.config import ty_config
from tygenie.widgets.input import TagValueInput


class AlertActionContainer(Widget):

    tag_value: reactive = reactive("", recompose=True)
    version: reactive = reactive(f"v{consts.VERSION}", recompose=True)
    saved_searches: reactive = reactive([], recompose=True)

    async def watch_tag_value(self):
        input = self.query_one("#tag_alert", Button)
        input.label = f"Tag alert '{self.tag_value}'"
        await self.recompose()

    async def watch_version(self):
        self.border_subtitle = self.version
        await self.recompose()

    async def watch_saved_searches(self):
        try:
            select = self.query_one("#saved_searches", Select)
            select.set_options(self.saved_searches)
            await self.recompose()
        except NoMatches:
            pass

    def __init__(self, tag_value: str = "", **kwargs) -> None:
        super().__init__(**kwargs)
        if tag_value == "":
            tag_value = ty_config.tygenie.get("default_tag", "")

        self.set_reactive(AlertActionContainer.tag_value, tag_value)

    def compose(self) -> ComposeResult:
        self.border_title = "Actions"
        with Horizontal(id="alert_action_horizontal_container"):
            yield TagValueInput(id="tag_value_container")
            yield Button(
                label=f"Tag alert '{self.tag_value}'", name="tag_alert", id="tag_alert"
            )
            yield Button(
                label="Open in webbrowser",
                name="open_in_webbrowser",
                id="open_in_webbrowser",
            )
            yield Button(label="Add note", name="add_note", id="add_note")
            if ty_config.tygenie.get("enable_saved_searches", True):
                yield Select(
                    id="saved_searches",
                    options=self.saved_searches,
                    allow_blank=True,
                    prompt="Use saved search",
                )

    class OpenInBrowser(Message):
        """A message to indicate that we have to open selected alert in webbrowser"""

    class AddNote(Message):
        """A message to indicate that we have to open selected alert alertmanager link"""

    class AddTag(Message):
        """A message to indicate that we have to open selected alert alertmanager link"""

    class CheckTygenieNewVersion(Message):
        """A message"""

    async def on_mount(self):
        self.send_check_tygenie_version_message()
        self.send_list_saved_searches()
        # Check every 6 hours for new version
        self.set_interval(6 * 60 * 60, self.send_check_tygenie_version_message)

    class ListSavedSearches(Message):
        """A message"""

    class LookupDataWithSearchIdentifier(Message):
        """A message"""

        def __init__(self, search_identifier: str):
            super().__init__()
            self.search_identifier = search_identifier

    @work(exclusive=False, exit_on_error=True, thread=True)
    def send_check_tygenie_version_message(self):
        self.post_message(self.CheckTygenieNewVersion())

    @work(exclusive=False, exit_on_error=True, thread=True)
    def send_list_saved_searches(self):
        self.post_message(self.ListSavedSearches())

    def on_button_pressed(self, event: Button.Pressed):
        if event.button.name == "open_in_webbrowser":
            self.post_message(self.OpenInBrowser())
        if event.button.name == "add_note":
            self.post_message(self.AddNote())
        if event.button.name == "tag_alert":
            self.post_message(self.AddTag())

    @on(TagValueInput.TagValueChange)
    def update_tag_value(self, message: TagValueInput.TagValueChange):
        self.tag_value = message.label

    @on(CheckTygenieNewVersion)
    async def check_tygenie_new_version(self):
        ty_logger.logger.log("Checking latest Tygenie release...")
        try:
            async with httpx.AsyncClient() as client:
                data = await client.get(
                    url="https://api.github.com/repos/ovh/tygenie/releases/latest"
                )
                latest_version = re.sub("^v", "", data.json()["name"])

            if semver.compare(consts.VERSION, latest_version) < 0:
                ty_logger.logger.log(f"New version found: {latest_version}")
                self.version = f"v{consts.VERSION} - [$text-success][r]v{latest_version} available![/][/]"
        except Exception:
            pass

    @on(ListSavedSearches)
    async def list_saved_searches(self):
        ty_logger.logger.log("List saved searches ...")
        try:
            saved_searches = await opsgenie.client.list_saved_searches()

            self.saved_searches = [
                (s.name, s.id)
                for s in sorted(saved_searches.data, key=lambda x: x.name.lower())
            ]
        except Exception as e:
            ty_logger.logger.log(f"[EXCEPTION] {e}")
            pass

    def on_select_changed(self, message: Message):
        try:
            # let's find the select option
            search_identifier = [
                i for i in self.saved_searches if i[1] == message.value
            ][0]
            self.post_message(
                self.LookupDataWithSearchIdentifier(search_identifier=search_identifier)
            )
        except Exception as e:
            ty_logger.logger.log(f"[EXCEPTION] {e}")
            pass



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notification_action_type import NotificationActionType
from ..types import UNSET, Unset

T = TypeVar("T", bound="NotificationRuleMeta")


@_attrs_define
class NotificationRuleMeta:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        action_type (Union[Unset, NotificationActionType]): Type of the action that notification rule will have
        order (Union[Unset, int]):
        enabled (Union[Unset, bool]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    action_type: Union[Unset, NotificationActionType] = UNSET
    order: Union[Unset, int] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        action_type: Union[Unset, str] = UNSET
        if not isinstance(self.action_type, Unset):
            action_type = self.action_type.value

        order = self.order

        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if action_type is not UNSET:
            field_dict["actionType"] = action_type
        if order is not UNSET:
            field_dict["order"] = order
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _action_type = d.pop("actionType", UNSET)
        action_type: Union[Unset, NotificationActionType]
        if isinstance(_action_type, Unset):
            action_type = UNSET
        else:
            action_type = NotificationActionType(_action_type)

        order = d.pop("order", UNSET)

        enabled = d.pop("enabled", UNSET)

        notification_rule_meta = cls(
            id=id,
            name=name,
            action_type=action_type,
            order=order,
            enabled=enabled,
        )

        notification_rule_meta.additional_properties = d
        return notification_rule_meta

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
